@@ -3,7 +3,7 @@ BEGIN {
   $Web::Machine::FSM::States::AUTHORITY = 'cpan:STEVAN';
 }
 {
-  $Web::Machine::FSM::States::VERSION = '0.05';
+  $Web::Machine::FSM::States::VERSION = '0.06';
 }
 # ABSTRACT: The States for Web Machine
 
@@ -116,30 +116,35 @@ sub _is_redirect {
     return;
 }
 
+sub _metadata {
+    my ($request) = @_;
+    return $request->env->{'web.machine.context'};
+}
+
 ## States
 
 $STATE_DESC{'b13'} = 'service_available';
 sub b13 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->service_available ? \&b12 : \503;
 }
 
 $STATE_DESC{'b12'} = 'known_method';
 sub b12 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     my $method = $request->method;
     (grep { $method eq $_ } @{ $resource->known_methods }) ? \&b11 : \501;
 }
 
 $STATE_DESC{'b11'} = 'uri_too_long';
 sub b11 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->uri_too_long( $request->uri ) ? \414 : \&b10;
 }
 
 $STATE_DESC{'b10'} = 'method_allowed';
 sub b10 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     my $method = $request->method;
     return \&b9 if grep { $method eq $_ } @{ $resource->allowed_methods };
     $response->header('Allow' => join ", " => @{ $resource->allowed_methods } );
@@ -148,13 +153,13 @@ sub b10 {
 
 $STATE_DESC{'b9'} = 'malformed_request';
 sub b9 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->malformed_request ? \400 : \&b8;
 }
 
 $STATE_DESC{'b8'} = 'is_authorized';
 sub b8 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     my $result = $resource->is_authorized( $request->header('Authorization') );
     # if we get back a status, then use it
     if ( is_status_code( $result ) ) {
@@ -178,13 +183,13 @@ sub b8 {
 
 $STATE_DESC{'b7'} = 'forbidden';
 sub b7 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->forbidden ? \403 : \&b6;
 }
 
 $STATE_DESC{'b6'} = 'content_headers_okay';
 sub b6 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
 
     # FIX-ME
     # there is a better way to do this,
@@ -205,19 +210,19 @@ sub b6 {
 
 $STATE_DESC{'b5'} = 'known_content_type';
 sub b5 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->known_content_type( $request->content_type ) ? \&b4 : \415;
 }
 
 $STATE_DESC{'b4'} = 'request_entity_too_large';
 sub b4 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->valid_entity_length( $request->content_length ) ? \&b3 : \413;
 }
 
 $STATE_DESC{'b3'} = 'method_is_options';
 sub b3 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     if ( $request->method eq 'OPTIONS' ) {
         $response->headers( $resource->options );
         return \200;
@@ -227,7 +232,8 @@ sub b3 {
 
 $STATE_DESC{'c3'} = 'accept_header_exists';
 sub c3 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
     if ( !$request->header('Accept') ) {
         $metadata->{'Content-Type'} = create_header( MediaType => (
             pair_key( $resource->content_types_provided->[0] )
@@ -239,7 +245,8 @@ sub c3 {
 
 $STATE_DESC{'c4'} = 'acceptable_media_type_available';
 sub c4 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
 
     my @types = map { pair_key( $_ ) } @{ $resource->content_types_provided };
 
@@ -253,14 +260,15 @@ sub c4 {
 
 $STATE_DESC{'d4'} = 'accept_language_header_exists';
 sub d4 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     (not $request->header('Accept-Language')) ? \&e5 : \&d5;
 }
 
 
 $STATE_DESC{'d5'} = 'accept_language_choice_available';
 sub d5 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
 
     if ( my $language = choose_language( $resource->languages_provided, $request->header('Accept-Language') ) ) {
         $metadata->{'Language'} = $language;
@@ -274,13 +282,14 @@ sub d5 {
 
 $STATE_DESC{'e5'} = 'accept_charset_exists';
 sub e5 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     (not $request->header('Accept-Charset')) ? \&f6 : \&e6;
 }
 
 $STATE_DESC{'e6'} = 'accept_charset_choice_available';
 sub e6 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
 
     if ( my $charset = choose_charset( $resource->charsets_provided, $request->header('Accept-Charset') ) ) {
         # handle the short circuit here ...
@@ -294,7 +303,8 @@ sub e6 {
 $STATE_DESC{'f6'} = 'accept_encoding_exists';
 # (also, set content-type header here, now that charset is chosen)
 sub f6 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
     if ( my $charset = $metadata->{'Charset'} ) {
         # Add the charset to the content type now ...
         $metadata->{'Content-Type'}->add_param( 'charset' => $charset );
@@ -319,7 +329,8 @@ sub f6 {
 
 $STATE_DESC{'f7'} = 'accept_encoding_choice_available';
 sub f7 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
 
     if ( my $encoding = choose_encoding( $resource->encodings_provided, $request->header('Accept-Encoding') ) ) {
         $response->header( 'Content-Encoding' => $encoding ) unless $encoding eq 'identity';
@@ -332,7 +343,7 @@ sub f7 {
 
 $STATE_DESC{'g7'} = 'resource_exists';
 sub g7 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
 
     # NOTE:
     # set Vary header here since we are
@@ -352,19 +363,19 @@ sub g7 {
 
 $STATE_DESC{'g8'} = 'if_match_exists';
 sub g8 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->header('If-Match') ? \&g9 : \&h10;
 }
 
 $STATE_DESC{'g9'} = 'if_match_is_wildcard';
 sub g9 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     _unquote_header( $request->header('If-Match') ) eq "*" ? \&h10 : \&g11;
 }
 
 $STATE_DESC{'g11'} = 'etag_in_if_match_list';
 sub g11 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     my @etags = map { _unquote_header( $_ ) } split /\s*\,\s*/ => $request->header('If-Match');
     my $etag  = $resource->generate_etag;
     (grep { $etag eq $_ } @etags) ? \&h10 : \412;
@@ -372,19 +383,20 @@ sub g11 {
 
 $STATE_DESC{'h7'} = 'if_match_exists_and_if_match_is_wildcard';
 sub h7 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     ($request->header('If-Match') && _unquote_header( $request->header('If-Match') ) eq "*") ? \412 : \&i7;
 }
 
 $STATE_DESC{'h10'} = 'if_unmodified_since_exists';
 sub h10 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->header('If-Unmodified-Since') ? \&h11 : \&i12;
 }
 
 $STATE_DESC{'h11'} = 'if_unmodified_since_is_valid_date';
 sub h11 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
     if ( my $date = $request->header('If-Unmodified-Since') ) {
         $metadata->{'If-Unmodified-Since'} = $date;
         return \&h12;
@@ -394,7 +406,8 @@ sub h11 {
 
 $STATE_DESC{'h12'} = 'last_modified_is_greater_than_if_unmodified_since';
 sub h12 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
     defined $resource->last_modified
         &&
     ($resource->last_modified->epoch > $metadata->{'If-Unmodified-Since'}->epoch)
@@ -403,7 +416,7 @@ sub h12 {
 
 $STATE_DESC{'i4'} = 'moved_permanently';
 sub i4 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     if ( my $uri = $resource->moved_permanently ) {
         if ( is_status_code( $uri ) ) {
             return $uri;
@@ -416,25 +429,25 @@ sub i4 {
 
 $STATE_DESC{'i7'} = 'method_is_put';
 sub i7 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->method eq 'PUT' ? \&i4 : \&k7
 }
 
 $STATE_DESC{'i12'} = 'if_none_match_exists';
 sub i12 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->header('If-None-Match') ? \&i13 : \&l13
 }
 
 $STATE_DESC{'i13'} = 'if_none_match_is_wildcard';
 sub i13 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->header('If-None-Match') eq "*" ? \&j18 : \&k13
 }
 
 $STATE_DESC{'j18'} = 'method_is_get_or_head';
 sub j18 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->method eq 'GET' || $request->method eq 'HEAD'
         ? _handle_304( $resource, $response )
         : \412
@@ -442,7 +455,7 @@ sub j18 {
 
 $STATE_DESC{'k5'} = 'moved_permanently';
 sub k5 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     if ( my $uri = $resource->moved_permanently ) {
         if ( is_status_code( $uri ) ) {
             return $uri;
@@ -455,13 +468,13 @@ sub k5 {
 
 $STATE_DESC{'k7'} = 'previously_existed';
 sub k7 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->previously_existed ? \&k5 : \&l7;
 }
 
 $STATE_DESC{'k13'} = 'etag_in_if_none_match';
 sub k13 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     my @etags = map { _unquote_header( $_ ) } split /\s*\,\s*/ => $request->header('If-None-Match');
     my $etag  = $resource->generate_etag;
     (grep { $etag eq $_ } @etags) ? \&j18 : \&l13;
@@ -469,7 +482,7 @@ sub k13 {
 
 $STATE_DESC{'l5'} = 'moved_temporarily';
 sub l5 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     if ( my $uri = $resource->moved_temporarily ) {
         if ( is_status_code( $uri ) ) {
             return $uri;
@@ -482,19 +495,20 @@ sub l5 {
 
 $STATE_DESC{'l7'} = 'method_is_post';
 sub l7 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->method eq 'POST' ? \&m7 : \404
 }
 
 $STATE_DESC{'l13'} = 'if_modified_since_exists';
 sub l13 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->header('If-Modified-Since') ? \&l14 : \&m16
 }
 
 $STATE_DESC{'l14'} = 'if_modified_since_is_valid_date';
 sub l14 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
     if ( my $date = $request->header('If-Modified-Since') ) {
         $metadata->{'If-Modified-Since'} = $date;
         return \&l15;
@@ -504,13 +518,15 @@ sub l14 {
 
 $STATE_DESC{'l15'} = 'if_modified_since_greater_than_now';
 sub l15 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
     ($metadata->{'If-Modified-Since'}->epoch > (scalar time)) ? \&m16 : \&l17;
 }
 
 $STATE_DESC{'l17'} = 'last_modified_is_greater_than_if_modified_since';
 sub l17 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
     defined $resource->last_modified
         &&
     ($resource->last_modified->epoch > $metadata->{'If-Modified-Since'}->epoch)
@@ -519,43 +535,43 @@ sub l17 {
 
 $STATE_DESC{'m5'} = 'method_is_post';
 sub m5 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->method eq 'POST' ? \&n5 : \410
 }
 
 $STATE_DESC{'m7'} = 'allow_post_to_missing_resource';
 sub m7 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->allow_missing_post ? \&n11 : \404
 }
 
 $STATE_DESC{'m16'} = 'method_is_delete';
 sub m16 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->method eq 'DELETE' ? \&m20 : \&n16
 }
 
 $STATE_DESC{'m20'} = 'delete_enacted_immediately';
 sub m20 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->delete_resource ? \&m20b : \500
 }
 
 $STATE_DESC{'m20b'} = 'did_delete_complete';
 sub m20b {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->delete_completed ? \&o20 : \202
 }
 
 $STATE_DESC{'n5'} = 'allow_post_to_missing_resource';
 sub n5 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->allow_missing_post ? \&n11 : \410
 }
 
 $STATE_DESC{'n11'} = 'redirect';
 sub n11 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     if ( $resource->post_is_create ) {
         my $uri = $resource->create_path;
         confess "Create Path Nil" unless $uri;
@@ -585,7 +601,7 @@ sub n11 {
         my $result = $resource->process_post;
         if ( $result ) {
             return $result if is_status_code( $result );
-            encode_body_if_set( $resource, $response, $metadata );
+            encode_body_if_set( $resource, $response );
         }
         else {
             confess "Process Post Invalid";
@@ -606,13 +622,13 @@ sub n11 {
 
 $STATE_DESC{'n16'} = 'method_is_post';
 sub n16 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->method eq 'POST' ? \&n11 : \&o16
 }
 
 $STATE_DESC{'o14'} = 'in_conflict';
 sub o14 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     return \409 if $resource->is_conflict;
 
     my $handler = _get_acceptable_content_type_handler( $resource, $request );
@@ -626,13 +642,14 @@ sub o14 {
 
 $STATE_DESC{'o16'} = 'method_is_put';
 sub o16 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $request->method eq 'PUT' ? \&o14 : \&o18;
 }
 
 $STATE_DESC{'o18'} = 'multiple_representations';
 sub o18 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
+    my $metadata = _metadata($request);
     if ( $request->method eq 'GET' || $request->method eq 'HEAD' ) {
         _add_caching_headers( $resource, $response );
 
@@ -648,8 +665,13 @@ sub o18 {
         return $result if is_status_code( $result );
 
         unless($request->method eq 'HEAD') {
-            $response->body( $result );
-            encode_body( $resource, $response, $metadata );
+            if (ref($result) eq 'CODE') {
+                $request->env->{'web.machine.streaming_push'} = $result;
+            }
+            else {
+                $response->body( $result );
+            }
+            encode_body( $resource, $response );
         }
         return \&o18b;
     }
@@ -661,19 +683,19 @@ sub o18 {
 
 $STATE_DESC{'o18b'} = 'multiple_choices';
 sub o18b {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $resource->multiple_choices ? \300 : \200;
 }
 
 $STATE_DESC{'o20'} = 'response_body_includes_entity';
 sub o20 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     $response->body ? \&o18 : \204;
 }
 
 $STATE_DESC{'p3'} = 'in_conflict';
 sub p3 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     return \409 if $resource->is_conflict;
 
     my $handler = _get_acceptable_content_type_handler( $resource, $request );
@@ -687,7 +709,7 @@ sub p3 {
 
 $STATE_DESC{'p11'} = 'new_resource';
 sub p11 {
-    my ($resource, $request, $response, $metadata) = @_;
+    my ($resource, $request, $response) = @_;
     (not $response->header('Location')) ? \&o20 : \201
 }
 
@@ -703,7 +725,7 @@ Web::Machine::FSM::States - The States for Web Machine
 
 =head1 VERSION
 
-version 0.05
+version 0.06
 
 =head1 SYNOPSIS
 
